"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var token_1 = require("../../../token");
var spelledOutNumbers_1 = require("../spelledOutNumbers");
var dayOfMonth = new RegExp(Object.keys(spelledOutNumbers_1.default)
    .map(function (item) { return "^" + item + "(th)?$"; })
    .concat('^\\d+(st|nd|rd|th)?$')
    .join('|'));
var dayOfMonthComposite = /^(twenty|thirty)[ -](\w+)$/;
var extractSingle = function (token) {
    var isNumber = token.match(/^\d+/);
    if (isNumber) {
        return new token_1.TokenDayOfMonth(Number(isNumber[0]));
    }
    // e.g. thirteenth => thirteen
    var key = token.replace(/th$/, '');
    return new token_1.TokenDayOfMonth(spelledOutNumbers_1.default[key]);
};
var extractComposite = function (token) {
    var match = token.match(dayOfMonthComposite);
    if (match === null) {
        throw new Error("Token " + token + " has shown to be a day-of-month-composite but cannot be parsed as such");
    }
    var computed = match
        .slice(1, 3)
        .map(function (part) { return spelledOutNumbers_1.default[part]; })
        .reduce(function (result, el) { return result + el; }, 0);
    return new token_1.TokenDayOfMonth(computed);
};
var EnglishDayOfMonthDetector = /** @class */ (function () {
    function EnglishDayOfMonthDetector() {
    }
    EnglishDayOfMonthDetector.prototype.match = function (token) {
        return dayOfMonth.test(token) || dayOfMonthComposite.test(token);
    };
    EnglishDayOfMonthDetector.prototype.extract = function (token) {
        return dayOfMonth.test(token)
            ? extractSingle(token)
            : extractComposite(token);
    };
    return EnglishDayOfMonthDetector;
}());
exports.default = EnglishDayOfMonthDetector;
//# sourceMappingURL=dayOfMonthDetector.js.map