"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var date_fns_1 = require("date-fns");
var parser_1 = require("./parser");
var token_1 = require("./token");
var english_1 = require("./locale/english");
// we don't try to compensate for missing attachments
var isComplete = function (tokens) {
    return tokens
        .filter(function (token) { return token.requiredAnchors().length > 0; })
        .every(function (tokenA) {
        return !!tokens.find(function (tokenB) { return tokenA.requiredAnchors().includes(tokenB.type); });
    });
};
var complete = function (source, target, reverse) {
    // 1. look for missing anchors
    var missingAnchors = source.filter(function (token) { return token.requiredAnchors().length > 0; });
    // 2. for each token, find its anchor and inject into the array right next to it
    // Reverse the search if we are completing the right from the left
    var searchIn = reverse ? target.slice().reverse() : target;
    missingAnchors.forEach(function (incomplete) {
        var position = source.indexOf(incomplete);
        var anchor = searchIn.find(function (token) {
            return incomplete.requiredAnchors().includes(token.type);
        });
        if (!anchor) {
            throw new Error("Cannot complete " + JSON.stringify(incomplete) + " from " + JSON.stringify(source) + " using " + JSON.stringify(target));
        }
        source.splice(position, 0, Object.assign(Object.create(Object.getPrototypeOf(anchor)), __assign({}, anchor, { attachments: [] })));
    });
};
var split = function (tokens, index) {
    var left = tokens.slice(0, index);
    var right = tokens.slice(index + 1);
    // sometimes anchors will be only on one side of the enumeration or range
    if (!isComplete(left)) {
        complete(left, right, false);
    }
    if (!isComplete(right)) {
        complete(right, left, true);
    }
    return [left, right];
};
var compute = function (tokens, today) {
    if (today === void 0) { today = new Date(); }
    if (tokens.length === 0) {
        return [];
    }
    // 1. split by emumeration
    var splitByEnumeration = tokens.findIndex(function (token) { return token.type === token_1.TokenTypes.ENUMERATION; });
    if (splitByEnumeration > -1) {
        var _a = split(tokens, splitByEnumeration), left = _a[0], right = _a[1];
        return _.flatMap([compute(left, today), compute(right, today)]);
    }
    // 2. split by range
    var splitByRange = tokens.findIndex(function (token) { return token.type === token_1.TokenTypes.RANGE; });
    if (splitByRange > -1) {
        var _b = split(tokens, splitByRange), left = _b[0], right = _b[1];
        // left and right equation of a range will always compute down to a date
        return date_fns_1.eachDay(compute(left, today)[0], compute(right, today)[0]);
    }
    // attach every token to its anchor
    tokens.forEach(function (token, idx) {
        // we can't filter, need to preserve the index in the original sequence
        // so that we could do a search for the nearest anchor candidate
        if (token.willAttachTo().length > 0) {
            var anchor = null;
            // how far back and forward we need to look
            var reach = Math.max(idx, tokens.length - 1 - idx);
            for (var step = 1; !anchor && step <= reach; step += 1) {
                // look behind and ahead and discard undefineds if we get out of boundary
                // give more priority to the token behind
                // ToDo: we probably need to give locales the control over what direction takes precedence?
                anchor = [tokens[idx - step], tokens[idx + step]]
                    .filter(Boolean)
                    .find(function (candidate) { return token.willAttachTo().includes(candidate.type); });
            }
            if (anchor) {
                anchor.attach(token);
            }
        }
    });
    // ToDo: do we need to inject a Year Direction token?
    // only compute top level tokes that didn't attach anywhere
    var heads = tokens.filter(function (token) { return !token.anchor; });
    var dates = _.flatMap(heads, function (head) { return head.compute(today); });
    return dates;
};
var NER = /** @class */ (function () {
    function NER(locale) {
        if (locale === void 0) { locale = new english_1.EnglishLocale(); }
        this.locale = locale;
    }
    NER.prototype.recognize = function (utterance, today) {
        if (today === void 0) { today = new Date(); }
        if (!utterance) {
            return [];
        }
        var sequence = parser_1.parse(utterance, this.locale);
        return compute(sequence.tokens.filter(function (token) { return token.type !== token_1.TokenTypes.UNDEFINED; }), today);
    };
    return NER;
}());
exports.NER = NER;
//# sourceMappingURL=ner.js.map