"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var english_1 = require("../../locale/english");
var parser_1 = require("../../parser");
var token_1 = require("../../token");
var locale = new english_1.EnglishLocale();
test('Parser should parse properly formatted dates', function () {
    var tokenized = parser_1.parse('12/15/18', locale).raw(false);
    var expected = [
        {
            token: '12/15/18',
            type: token_1.TokenTypes.DATE
        }
    ];
    expect(tokenized).toMatchObject(expected);
});
test('Parser should parse months and single dates', function () {
    var tokenized = parser_1.parse('in NYC on April the 5th', locale).raw(false);
    var expected = [
        {
            token: 4,
            type: token_1.TokenTypes.MONTH
        },
        {
            token: 5,
            type: token_1.TokenTypes.DAY_OF_MONTH
        }
    ];
    expect(tokenized).toMatchObject(expected);
});
test('Parser should see ranges and enumerations', function () {
    var tokenized = parser_1.parse('in NYC 5/15 - 5/18 and 5/20', locale).raw(false);
    var expected = [
        {
            token: '5/15',
            type: token_1.TokenTypes.DATE
        },
        {
            token: '-',
            type: token_1.TokenTypes.RANGE
        },
        {
            token: '5/18',
            type: token_1.TokenTypes.DATE
        },
        {
            token: 'and',
            type: token_1.TokenTypes.ENUMERATION
        },
        {
            token: '5/20',
            type: token_1.TokenTypes.DATE
        }
    ];
    expect(tokenized).toMatchObject(expected);
});
test('Parser should see days of week and direction', function () {
    var tokenized = parser_1.parse('was in London last week Wed through Friday', locale).raw(false);
    var expected = [
        {
            token: -1,
            type: token_1.TokenTypes.DIRECTION_WEEK
        },
        {
            token: 3,
            type: token_1.TokenTypes.DAY_OF_WEEK
        },
        {
            token: 'through',
            type: token_1.TokenTypes.RANGE
        },
        {
            token: 5,
            type: token_1.TokenTypes.DAY_OF_WEEK
        }
    ];
    expect(tokenized).toMatchObject(expected);
});
test('Parser should see spelled out days', function () {
    var tokenized = parser_1.parse('in new york on may twenty fifth', locale).raw(false);
    var expected = [
        {
            token: 5,
            type: token_1.TokenTypes.MONTH
        },
        {
            token: 25,
            type: token_1.TokenTypes.DAY_OF_MONTH
        }
    ];
    expect(tokenized).toMatchObject(expected);
});
test('Parser should understand named days', function () {
    var tokenized = parser_1.parse('in London today and tomorrow', locale).raw(false);
    var expected = [
        {
            token: 0,
            type: token_1.TokenTypes.NAMED_DAY
        },
        {
            token: 'and',
            type: token_1.TokenTypes.ENUMERATION
        },
        {
            token: 1,
            type: token_1.TokenTypes.NAMED_DAY
        }
    ];
    expect(tokenized).toMatchObject(expected);
});
test('Parser should understand durations', function () {
    var tokenized = parser_1.parse('I will in Dubai next two weeks', locale).raw(false);
    var expected = [
        {
            token: 1,
            type: token_1.TokenTypes.DIRECTION_WEEK
        },
        {
            token: 'two weeks',
            type: token_1.TokenTypes.DURATION
        }
    ];
    expect(tokenized).toMatchObject(expected);
});
test('Tokenizer shoud understand starting anchors', function () {
    var tokenized = parser_1.parse('in Australlia for 2 weeks starting Thu', locale).raw(false);
    var expected = [
        {
            token: '2 weeks',
            type: token_1.TokenTypes.DURATION
        },
        {
            token: 'starting',
            type: token_1.TokenTypes.ANCHOR_START
        },
        {
            token: 4,
            type: token_1.TokenTypes.DAY_OF_WEEK
        }
    ];
    expect(tokenized).toMatchObject(expected);
});
[
    'will be in Dallas all next week',
    'will be in Dallas all week next week'
].forEach(function (token) {
    return test("Parser should understand a combination of duration and direction in [" + token + "]", function () {
        var tokenized = parser_1.parse(token, locale).raw(false);
        var expected = [
            {
                token: 'all week',
                type: token_1.TokenTypes.DURATION
            },
            {
                token: 1,
                type: token_1.TokenTypes.DIRECTION_WEEK
            }
        ];
        expect(tokenized).toMatchObject(expected);
    });
});
test('Parser should understand quite complex expressions', function () {
    var parsed = parser_1.parse('Tue through Friday next week and then again Wed and Thu the week after next', locale).raw(false);
    var expected = [
        {
            token: 2,
            type: token_1.TokenTypes.DAY_OF_WEEK
        },
        {
            token: 'through',
            type: token_1.TokenTypes.RANGE
        },
        {
            token: 5,
            type: token_1.TokenTypes.DAY_OF_WEEK
        },
        {
            token: 1,
            type: token_1.TokenTypes.DIRECTION_WEEK
        },
        {
            token: 'and',
            type: token_1.TokenTypes.ENUMERATION
        },
        {
            token: 3,
            type: token_1.TokenTypes.DAY_OF_WEEK
        },
        {
            token: 'and',
            type: token_1.TokenTypes.ENUMERATION
        },
        {
            token: 4,
            type: token_1.TokenTypes.DAY_OF_WEEK
        },
        {
            token: 2,
            type: token_1.TokenTypes.DIRECTION_WEEK
        }
    ];
    expect(parsed).toMatchObject(expected);
});
//# sourceMappingURL=parser.test.js.map