"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var date_fns_1 = require("date-fns");
var tokenDate_1 = require("../tokenDate");
var types_1 = require("../types");
test('Token [DATE] has correct type and behavior settings', function () {
    var token = new tokenDate_1.default('5/15');
    expect(token.type).toEqual(types_1.default.DATE);
    expect(token.willAnchor()).toMatchObject([]);
    expect(token.willAttachTo()).toMatchObject([]);
    expect(token.isComplete()).toBeTruthy();
});
['5/15', '05/15/2018', '2018-05-15'].forEach(function (date) {
    return test("Token [DATE] properly computes itself for [" + date + "]", function () {
        var token = new tokenDate_1.default(date);
        expect(token.compute()).toEqual(date_fns_1.startOfDay(new Date('05/15/2018')));
    });
});
test('Token [DATE] can understand dates in DD/MM format', function () {
    var token = new tokenDate_1.default('2/3/18', false);
    expect(token.compute()).toEqual(date_fns_1.parse('2018-03-02'));
});
['not a date', '25', '28-28-115', '2.3.3', '13/13/2018'].forEach(function (date) {
    return test("Token [DATE] will throw an Error on [" + date + "]", function () {
        expect(function () {
            var token = new tokenDate_1.default(date);
        }).toThrowError();
    });
});
//# sourceMappingURL=tokenDate.test.js.map